@@ -1,6 +1,6 @@
 /* vim: set expandtab ts=8 sw=4: */
 
-/*  $Id: panel-config.c 22697 2006-08-09 18:36:00Z jasper $
+/*  $Id: panel-config.c 24092 2006-12-13 19:07:59Z jasper $
  *
  *  Copyright © 2005 Jasper Huijsmans <jasper@xfce.org>
  *
@@ -33,9 +33,14 @@
 #include <gdk/gdkx.h>
 #include <gtk/gtkenums.h>
 
+#ifdef HAVE_SYS_MMAN_H
+#include <sys/mman.h>   
+#endif
+
 #include <libxfce4util/libxfce4util.h>
 #include <libxfce4panel/xfce-panel-macros.h>
 #include <libxfce4panel/xfce-panel-convenience.h>
+#include <libxfce4panel/xfce-panel-item-iface.h>
 
 #include "panel-config.h"
 #include "panel-private.h"
@@ -51,7 +56,8 @@
 
 static GPtrArray *config_parse_file (const char *filename);
 
-static gboolean config_save_to_file (GPtrArray *panels, const char *filename);
+static gboolean config_save_to_file (GPtrArray *panels, 
+                                     const char *filename);
 
 
 /* fallback panel */
@@ -60,7 +66,7 @@ static GPtrArray *
 create_fallback_panel_array (void)
 {
     GPtrArray *array;
-    Panel *panel;
+    Panel     *panel;
 
     DBG ("No suitable panel configuration was found.");
     
@@ -77,36 +83,20 @@ create_fallback_panel_array (void)
     return array;
 }
 
-/* parsing the config file */
-
-static GPtrArray *
-create_panel_array_from_config (const char *file)
-{
-    GPtrArray *array;
-
-    array = config_parse_file (file);
-
-    if (!array)
-        array = create_fallback_panel_array ();
-
-    return array;
-}
-
 /* public API */
 
 GPtrArray *
 panel_config_create_panels (void)
 {
-    char *file = NULL;
-    GPtrArray *array = NULL;
-    gboolean use_user_config;
+    char       *file  = NULL;
+    GPtrArray  *array = NULL;
+    const char *path  = "xfce4" G_DIR_SEPARATOR_S 
+                        "panel" G_DIR_SEPARATOR_S
+                        "panels.xml";
 
-    use_user_config = xfce_allow_panel_customization ();
-
-    if (G_UNLIKELY (!use_user_config))
+    if (G_UNLIKELY (!xfce_allow_panel_customization ()))
     {
-        file = g_build_filename (SYSCONFDIR, "xdg", "xfce4", "panel",
-                                 "panels.xml", NULL);
+        file = g_build_filename (SYSCONFDIR, "xdg", path, NULL);
 
         if (!g_file_test (file, G_FILE_TEST_IS_REGULAR))
         {
@@ -117,29 +107,28 @@ panel_config_create_panels (void)
     }
     else
     {
-        char *path = g_build_filename ("xfce4", "panel", "panels.xml", NULL);
-
         file = xfce_resource_lookup (XFCE_RESOURCE_CONFIG, path);
 
-        g_free (path);
-    }
-
-    if (G_UNLIKELY (!file && use_user_config))
-    {
-        file = g_build_filename (SYSCONFDIR, "xdg", "xfce4", "panel",
-                                 "panels.xml", NULL);
-
-        if (!g_file_test (file, G_FILE_TEST_IS_REGULAR))
+        if (G_UNLIKELY (!file))
         {
-            g_free (file);
+            file = g_build_filename (SYSCONFDIR, path, NULL);
 
-            file = NULL;
+            if (!g_file_test (file, G_FILE_TEST_IS_REGULAR))
+            {
+                g_free (file);
+
+                file = NULL;
+            }
         }
     }
-    
+
     if (file)
-        array = create_panel_array_from_config (file);
-    else
+    {
+        array = config_parse_file (file);
+        g_free (file);
+    }
+    
+    if (!array)
         array = create_fallback_panel_array ();
 
     DBG ("Successfully configured %d panel(s).", array->len);
@@ -150,25 +139,22 @@ panel_config_create_panels (void)
 gboolean
 panel_config_save_panels (GPtrArray * panels)
 {
-    char *file = NULL;
     gboolean failed = FALSE;
-    gboolean use_user_config;
 
-    use_user_config = xfce_allow_panel_customization ();
-
-    if (use_user_config)
+    if (xfce_allow_panel_customization ())
     {
-        int i;
-        char *path;
-        
-        path = g_build_filename ("xfce4", "panel", "panels.xml", NULL);
+        int         i;
+        char       *file = NULL;
+        const char *path = "xfce4" G_DIR_SEPARATOR_S 
+                           "panel" G_DIR_SEPARATOR_S
+                           "panels.xml";
 
         file = xfce_resource_save_location (XFCE_RESOURCE_CONFIG, path, TRUE);
 
-        g_free (path);
-
         failed = !config_save_to_file (panels, file);
 
+        g_free (file);
+
         for (i = 0; i < panels->len; ++i)
         {
             Panel *panel = g_ptr_array_index (panels, i);
@@ -196,24 +182,23 @@ ParserState;
 typedef struct _ConfigParser ConfigParser;
 struct _ConfigParser
 {
-    GPtrArray *panels;
-    Panel *current_panel;
-    ParserState state;
+    GPtrArray   *panels;
+    Panel       *current_panel;
+    ParserState  state;
 
-    gboolean properties_set;
-    gboolean monitor_set;
+    gboolean     properties_set;
+    gboolean     monitor_set;
     
-    /* properties */
-    int size;
-    int monitor;
-    int screen_position;
-    int full_width;
-    int xoffset;
-    int yoffset;
-    int handle_style;
-    gboolean autohide;
-    int transparency;
-    gboolean activetrans;
+    int          size;
+    int          monitor;
+    int          screen_position;
+    int          full_width;
+    int          xoffset;
+    int          yoffset;
+    int          handle_style;
+    gboolean     autohide;
+    int          transparency;
+    gboolean     activetrans;
 };
 
 static void
@@ -236,7 +221,8 @@ init_properties (ConfigParser *parser)
 
 static void
 config_set_property (ConfigParser *parser, 
-                     const char *name, const char *value)
+                     const char   *name, 
+                     const char   *value)
 {
     g_return_if_fail (name != NULL && value != NULL);
 
@@ -286,17 +272,17 @@ config_set_property (ConfigParser *parser,
 }
 
 static void
-start_element_handler (GMarkupParseContext * context,
-                       const char *element_name,
-                       const char **attribute_names,
-                       const char **attribute_values,
-                       gpointer user_data, 
-                       GError ** error)
+start_element_handler (GMarkupParseContext  *context,
+                       const char           *element_name,
+                       const char          **attribute_names,
+                       const char          **attribute_values,
+                       gpointer              user_data, 
+                       GError              **error)
 {
     ConfigParser *parser = user_data;
-    char *name = NULL;
-    char *value = NULL;
-    int i = 0;
+    char         *name   = NULL;
+    char         *value  = NULL;
+    int           i      = 0;
 
     switch (parser->state)
     {
@@ -306,6 +292,7 @@ start_element_handler (GMarkupParseContext * context,
                 parser->state = PANELS;
             }
             break;
+
         case PANELS:
             if (strcmp (element_name, "panel") == 0)
             {
@@ -395,10 +382,10 @@ start_element_handler (GMarkupParseContext * context,
 }
 
 static void
-end_element_handler (GMarkupParseContext * context,
-                     const char *element_name,
-                     gpointer user_data, 
-                     GError ** error)
+end_element_handler (GMarkupParseContext  *context,
+                     const char           *element_name,
+                     gpointer              user_data, 
+                     GError              **error)
 {
     ConfigParser *parser = user_data;
 
@@ -407,10 +394,12 @@ end_element_handler (GMarkupParseContext * context,
         case START:
             g_warning ("end unexpected element: \"%s\"", element_name);
             break;
+
         case PANELS:
             if (strcmp ("panels", element_name) == 0)
                 parser->state = START;
             break;
+
         case PANEL:
             if (strcmp ("panel", element_name) == 0)
             {
@@ -435,18 +424,17 @@ end_element_handler (GMarkupParseContext * context,
                         parser->monitor = DefaultScreen (GDK_DISPLAY());
                     }
 
-                    
                     g_object_set (G_OBJECT (parser->current_panel),
-                                  "size", parser->size,
-                                  "monitor", parser->monitor,
+                                  "size",            parser->size,
+                                  "monitor",         parser->monitor,
                                   "screen-position", parser->screen_position,
-                                  "fullwidth", parser->full_width,
-                                  "xoffset", parser->xoffset,
-                                  "yoffset", parser->yoffset,
-                                  "handle-style", parser->handle_style,
-                                  "autohide", parser->autohide,
-                                  "transparency", parser->transparency,
-                                  "activetrans", parser->activetrans,
+                                  "fullwidth",       parser->full_width,
+                                  "xoffset",         parser->xoffset,
+                                  "yoffset",         parser->yoffset,
+                                  "handle-style",    parser->handle_style,
+                                  "autohide",        parser->autohide,
+                                  "transparency",    parser->transparency,
+                                  "activetrans",     parser->activetrans,
                                   NULL);
                 }
                 TIMER_ELAPSED(" ++ end properties");
@@ -478,16 +466,16 @@ static GMarkupParser markup_parser = {
 static GPtrArray *
 config_parse_file (const char *filename)
 {
-    GPtrArray *array = NULL;
-    char *contents;
-    GError *error;
+    GPtrArray           *array = NULL;
+    char                *contents;
+    GError              *error;
     GMarkupParseContext *context;
-    ConfigParser parser;
-    struct stat sb;
-    size_t bytes;
-    int fd, rc;
+    ConfigParser         parser;
+    struct stat          sb;
+    size_t               bytes;
+    int                  fd, rc;
 #ifdef HAVE_MMAP
-    void *addr;
+    void                *addr;
 #endif
 
     g_return_val_if_fail (filename != NULL && strlen (filename) > 0, NULL);
@@ -587,9 +575,8 @@ finished:
     {
         if (munmap (addr, sb.st_size) < 0)
         {
-            g_critical ("Unable to unmap file %s with contents for channel "
-                        "\"%s\": %s. This should not happen!",
-                        filename, channel_name, g_strerror (errno));
+            g_critical ("Unable to unmap file %s: %s. This should not happen!", 
+                        filename, g_strerror (errno));
         }
 
         contents = NULL;
@@ -618,8 +605,8 @@ gboolean
 config_save_to_file (GPtrArray *array, const char *filename)
 {
     FILE *fp;
-    char tmp_path[PATH_MAX];
-    int i;
+    char  tmp_path[PATH_MAX];
+    int   i;
 
     g_return_val_if_fail (array != NULL, FALSE);
     g_return_val_if_fail (filename != NULL || (strlen (filename) > 0), FALSE);
@@ -643,31 +630,34 @@ config_save_to_file (GPtrArray *array, const char *filename)
 
     for (i = 0; i < array->len; ++i)
     {
-        Panel *panel;
-        int size, monitor, screen_position, xoffset, yoffset, handle_style,
-            transparency, fullwidth, j;
-        gboolean autohide, activetrans;
-        XfcePanelItemConfig *configlist;
+        GList    *configlist, *l;
+        Panel    *panel;
+        int       size            = 0;
+        int       monitor         = 0;
+        int       screen_position = 0;
+        int       fullwidth       = 0;
+        int       xoffset         = 0;
+        int       yoffset         = 0;
+        int       handle_style    = 0;
+        int       transparency    = 0;
+        gboolean  autohide        = FALSE;
+        gboolean  activetrans     = FALSE;
         
         DBG ("Saving panel %d", i + 1);
 
         panel = g_ptr_array_index (array, i);
 
-        size = monitor = screen_position = xoffset = yoffset = 
-            transparency = fullwidth = 0;
-        autohide = activetrans = FALSE;
-
         g_object_get (G_OBJECT (panel),
-                      "size", &size,
-                      "monitor", &monitor,
+                      "size",            &size,
+                      "monitor",         &monitor,
                       "screen-position", &screen_position,
-                      "fullwidth", &fullwidth,
-                      "xoffset", &xoffset,
-                      "yoffset", &yoffset,
-                      "handle-style", &handle_style,
-                      "autohide", &autohide,
-                      "transparency", &transparency,
-                      "activetrans", &activetrans,
+                      "fullwidth",       &fullwidth,
+                      "xoffset",         &xoffset,
+                      "yoffset",         &yoffset,
+                      "handle-style",    &handle_style,
+                      "transparency",    &transparency,
+                      "autohide",        &autohide,
+                      "activetrans",     &activetrans,
                       NULL);
         
         /* grouping */
@@ -710,23 +700,22 @@ config_save_to_file (GPtrArray *array, const char *filename)
                      "\t\t<items>\n");
 
         /* panel items */
-        configlist = panel_get_item_config_list (panel);
+        configlist = panel_get_item_list (panel);
 
-        if (configlist)
+        for (l = configlist; l != NULL; l = l->next)
         {
-            for (j = 0; configlist[j].name != NULL; ++j)
-            {
-                fprintf (fp, "\t\t\t<item name=\"%s\" id=\"%s\"/>\n",
-                             configlist[j].name, configlist[j].id);
-            }
-            
-            g_free (configlist);
+            XfcePanelItem *item = l->data;
+
+            fprintf (fp, "\t\t\t<item name=\"%s\" id=\"%s\"/>\n",
+                         xfce_panel_item_get_name (item),
+                         xfce_panel_item_get_id   (item));
         }
+            
+        g_list_free (configlist);
 
         /* grouping */
         fprintf (fp, "\t\t</items>\n"
                      "\t</panel>\n");
-
     }
 
     /* closing */
